import { MigrationInterface, QueryRunner, Table, TableForeignKey } from "typeorm";
import { DB_TYPE } from "../env.defaults";

export class AddWebapiTokens1737453096674 implements MigrationInterface {
  name = "AddWebApiTokens1737453096674";

  public async up(queryRunner: QueryRunner): Promise<void> {
    const variableType_int = DB_TYPE == "mysql" ? "int" : "integer";

    await queryRunner.createTable(
      new Table({
        name: "webapi",
        columns: [
          { name: "id", type: variableType_int, isPrimary: true, isGenerated: true, generationStrategy: "increment" },
          { name: "token", type: "text", isUnique: true, isNullable: false },
          { name: "title", type: "varchar", isUnique: true, length: "255", isNullable: false },
          { name: "createdAt", type: "datetime", default: "CURRENT_TIMESTAMP(6)", isNullable: false },
          { name: "lastUsage", type: "datetime", isNullable: true, default: null },
          { name: "expiry", type: "date", isNullable: true, default: null },
        ],
      }),
      true
    );

    await queryRunner.createTable(
      new Table({
        name: "webapi_permission",
        columns: [
          { name: "webapiId", type: variableType_int, isPrimary: true, isNullable: false },
          { name: "permission", type: "varchar", length: "255", isPrimary: true, isNullable: false },
        ],
      }),
      true
    );

    await queryRunner.createForeignKey(
      "webapi_permission",
      new TableForeignKey({
        columnNames: ["webapiId"],
        referencedColumnNames: ["id"],
        referencedTableName: "webapi",
        onDelete: "CASCADE",
        onUpdate: "RESTRICT",
      })
    );
  }

  public async down(queryRunner: QueryRunner): Promise<void> {
    const table = await queryRunner.getTable("webapi_permission");
    const foreignKey = table.foreignKeys.find((fk) => fk.columnNames.indexOf("webapiId") !== -1);
    await queryRunner.dropForeignKey("webapi_permission", foreignKey);
    await queryRunner.dropTable("webapi_permission");
    await queryRunner.dropTable("webapi");
  }
}
